@@ -64,7 +64,7 @@
 #' @importFrom jsonlite fromJSON
 #' @importFrom preprocessCore normalize.quantiles
 #' @importFrom InterpretMSSpectrum findMAIN PlotSpec
-#' @importFrom xcms groupval
+#' @importFrom xcms groupval featureValues
 
 #' @references Broeckling CD, Afsar FA, Neumann S, Ben-Hur A, Prenni JE. RAMClust: a novel feature clustering method enables spectral-matching-based annotation for metabolomics data. Anal Chem. 2014 Jul 15;86(14):6812-7. doi: 10.1021/ac501530d.  Epub 2014 Jun 26. PubMed PMID: 24927477.
 #' @references Broeckling CD, Ganna A, Layer M, Brown K, Sutton B, Ingelsson E, Peers G, Prenni JE. Enabling Efficient and Confident Annotation of LC-MS Metabolomics Data through MS1 Spectrum and Time Prediction. Anal Chem. 2016 Sep 20;88(18):9226-34. doi: 10.1021/acs.analchem.6b02479. Epub 2016 Sep 8. PubMed PMID: 7560453.
@@ -72,7 +72,7 @@
 #' @author Corey Broeckling
 #' @export
 
-ramclustR<- function(  xcmsObj=NULL,
+ramclustR  <- function(xcmsObj=NULL,
                        ms=NULL, 
                        idmsms=NULL,
                        taglocation="filepaths",
@@ -137,7 +137,7 @@ ramclustR<- function(  xcmsObj=NULL,
                    ") was utilized to cluster features into spectra (Broeckling 2014).",
                    sep = "")  
   
-
+  
   ########
   # do some checks to make sure we have everything we need before proceeding
   if(is.null(xcmsObj) & is.null(ms))  {
@@ -173,10 +173,10 @@ ramclustR<- function(  xcmsObj=NULL,
     if(is.null(maxt)) maxt<-60
     
     history <- paste(history,
-                    " Feature data was input as .csv files with ramclustR parameter settings of ",
-                    " st = ", st,
-                    " sr = ", sr,
-                    " and maxt = ", maxt, ".", sep = ""
+                     " Feature data was input as .csv files with ramclustR parameter settings of ",
+                     " st = ", st,
+                     " sr = ", sr,
+                     " and maxt = ", maxt, ".", sep = ""
     )
     
     MSdata<-read.csv(file = ms, header=TRUE, check.names=FALSE)
@@ -210,13 +210,21 @@ ramclustR<- function(  xcmsObj=NULL,
     rm(rtmz)
   }
   
+  newXCMS <- FALSE
   ########
   # if xcms object as input, do this:
   if(!is.null(xcmsObj)){
-    if(!class(xcmsObj)=="xcmsSet")
-    {stop("xcmsObj must reference an object generated by XCMS, of class 'xcmsSet'")}
+    OK <- FALSE
     
-    if(is.null(st)) st<-round(median(xcmsObj@peaks[,"rtmax"]-xcmsObj@peaks[,"rtmin"])/2, digits=2)
+    if(class(xcmsObj)=="xcmsSet") {OK <- TRUE} 
+    if(class(xcmsObj) == "XCMSnExp") {
+      OK <- TRUE
+      newXCMS <- TRUE
+    }
+    if(!OK) {stop("xcmsObj must reference an object generated by XCMS, of class 'xcmsSet'")}
+    
+    if(is.null(st) & !newXCMS) st<-round(median(xcmsObj@peaks[,"rtmax"]-xcmsObj@peaks[,"rtmin"])/2, digits=2)
+    if(is.null(st) & newXCMS) st<-round(median(xcmsObj@msFeatureData$chromPeaks[,"rtmax"]-xcmsObj@msFeatureData$chromPeaks[,"rtmin"])/2, digits=2)
     if(is.null(sr)) sr<-0.5
     if(is.null(maxt)) maxt<-st*20
     
@@ -227,12 +235,21 @@ ramclustR<- function(  xcmsObj=NULL,
                      " and maxt = ", maxt, ".", sep = ""
     )
     
-    sampnames<-row.names(xcmsObj@phenoData)
-    data12<-groupval(xcmsObj, value="into")
+    if(!newXCMS) {sampnames<-row.names(xcmsObj@phenoData)}
+    if(newXCMS)  {sampnames <- xcmsObj@phenoData[[1]]}
+    if(!newXCMS) {data12<-groupval(xcmsObj, value="into")}
+    if(newXCMS) {data12<-featureValues(xcmsObj, value = "into")}
     
     if(taglocation=="filepaths" & !is.null(MStag)){ 
-      msfiles<-grep(MStag, xcmsObj@filepaths, ignore.case=TRUE)
-      msmsfiles<-grep(idMSMStag, xcmsObj@filepaths, ignore.case=TRUE)
+      if(!newXCMS) {
+        msfiles<-grep(MStag, xcmsObj@filepaths, ignore.case=TRUE)
+        msmsfiles<-grep(idMSMStag, xcmsObj@filepaths, ignore.case=TRUE)
+      }
+      if(newXCMS) {
+        msfiles<-grep(MStag, row.names(xcmsObj@phenoData), ignore.case=TRUE)
+        msmsfiles<-grep(idMSMStag, row.names(xcmsObj@phenoData), ignore.case=TRUE)
+      }
+      
       if(length(intersect(msfiles, msmsfiles)>0)) {stop("your MS and idMSMStag values do not generate unique file lists")}
       if(length(msfiles)!=length(msmsfiles))  {stop("the number of MS files must equal the number of MSMS files")}
       data1<-t(data12[,msfiles])
@@ -249,39 +266,63 @@ ramclustR<- function(  xcmsObj=NULL,
         batch   <- batch[msfiles]
       }
       
-      times<-round(xcmsObj@groups[,"rtmed"], digits=3)
+      if(!newXCMS) {times<-round(xcmsObj@groups[,"rtmed"], digits=3)}
+      if(newXCMS) {times<-round(xcmsObj@msFeatureData$featureDefinitions$rtmed, digits = 3)}
+      
       if(any(is.na(times))) {
         do<-which(is.na(times))
         for(x in 1:length(do)) {
-          times[do[x]]<-  as.numeric((xcmsObj@groups[do[x],"rtmin"]+ xcmsObj@groups[do[x],"rtmax"])/2)
+          if(!newXCMS) {
+            times[do[x]]<-  as.numeric((xcmsObj@groups[do[x],"rtmin"]+ xcmsObj@groups[do[x],"rtmax"])/2)
+          }
+          if(newXCMS) {
+            times[do[x]] <- as.numeric((xcmsObj@msFeatureData$featureDefinitions$rtmin[do[x]] + xcmsObj@msFeatureData$featureDefinitions$rtmax[do[x]])/2)
+          }
         }
       }
       # if(any(is.na(times))) {stop("na values still present")} else {print("NAs removed")}
-      mzs<-round(xcmsObj@groups[,"mzmed"], digits=4)
+      if(!newXCMS) {mzs<-round(xcmsObj@groups[,"mzmed"], digits=4)}
+      if(newXCMS) {mzs<-round(xcmsObj@msFeatureData$featureDefinitions$mzmed, digits = 4)}
+      
     } else {
       data1<-t(data12)
       msfiles<-1:nrow(data1)
       data2<-t(data12)
-      times<-round(xcmsObj@groups[,"rtmed"], digits=3)
-      mzs<-round(xcmsObj@groups[,"mzmed"], digits=4) 
+      
+      row.names(data1)<-sampnames[msfiles]
+      row.names(data2)<-sampnames[msfiles]
+      
+      if(!newXCMS) {times<-round(xcmsObj@groups[,"rtmed"], digits=3)}
+      if(newXCMS) {times<-round(xcmsObj@msFeatureData$featureDefinitions$rtmed, digits = 3)}
+      
       if(any(is.na(times))) {
         do<-which(is.na(times))
         for(x in 1:length(do)) {
-          times[do[x]]<-  as.numeric((xcmsObj@groups[do[x],"rtmin"]+ xcmsObj@groups[do[x],"rtmax"])/2)
+          if(!newXCMS) {
+            times[do[x]]<-  as.numeric((xcmsObj@groups[do[x],"rtmin"]+ xcmsObj@groups[do[x],"rtmax"])/2)
+          }
+          if(newXCMS) {
+            times[do[x]] <- as.numeric((xcmsObj@msFeatureData$featureDefinitions$rtmin[do[x]] + xcmsObj@msFeatureData$featureDefinitions$rtmax[do[x]])/2)
+          }
         }
       }
+      # if(any(is.na(times))) {stop("na values still present")} else {print("NAs removed")}
+      if(!newXCMS) {mzs<-round(xcmsObj@groups[,"mzmed"], digits=4)}
+      if(newXCMS) {mzs<-round(xcmsObj@msFeatureData$featureDefinitions$mzmed, digits = 4)}
       
-      if (normalize == "batch.qc") {
-        qc2 <- qc[msfiles]
-        qc <- qc[msfiles]
-        order2 <- order[msfiles]
-        order <- order[msfiles]
-        batch2 <- batch[msfiles]
-        batch <- batch[msfiles]
-      }
-      
-      #if(any(is.na(times))) {stop("na values still present")} else {print("NAs removed")}
     }
+    
+    if (normalize == "batch.qc") {
+      qc2 <- qc[msfiles]
+      qc <- qc[msfiles]
+      order2 <- order[msfiles]
+      order <- order[msfiles]
+      batch2 <- batch[msfiles]
+      batch <- batch[msfiles]
+    }
+    
+    #if(any(is.na(times))) {stop("na values still present")} else {print("NAs removed")}
+    #}
   }
   
   ## if using batch.qc check for proper information
@@ -316,7 +357,7 @@ ramclustR<- function(  xcmsObj=NULL,
   ########
   # Optional normalization of data, either Total ion signal or quantile
   
-  cat(paste("  normalizing dataset", '\n'))
+  if(!normalize == "none") cat(paste("  normalizing dataset", '\n'))
   
   ## for batch.qc method, we will order the datasets first by batch and run order
   # backup <- data1
@@ -342,7 +383,7 @@ ramclustR<- function(  xcmsObj=NULL,
     batch2 <- ndf[,"batch"]
     qc2 <- ndf[,"qc"]
     order2 <- ndf[,"order"]
-
+    
   }
   
   data1raw <- data1
@@ -357,6 +398,7 @@ ramclustR<- function(  xcmsObj=NULL,
                      " Features were normalized to total signal using 'tic' normalization."
     )
   }
+  
   if(normalize=="quantile") {
     tmpnames1<-dimnames(data1)
     tmpnames2<-dimnames(data2)
@@ -570,6 +612,8 @@ ramclustR<- function(  xcmsObj=NULL,
   # data organization and parsing 
   # sort rt vector and data by retention time
   xcmsOrd<-order(times)
+  # cat(times[1:100], '\n')
+  # cat('dim data1:', dim(data1), '\n')
   data1<-data1[,order(times)]
   data2<-data2[,order(times)]
   mzs<-mzs[order(times)]
@@ -709,7 +753,7 @@ ramclustR<- function(  xcmsObj=NULL,
                    "The feature similarity matrix was clustered using fastcluster package heirarchical clustering method using the",
                    linkage, "method."
   )
-
+  
   gc()
   d<-Sys.time()    
   cat("fastcluster based clustering complete", '\n')
@@ -729,7 +773,7 @@ ramclustR<- function(  xcmsObj=NULL,
                    "minModuleSize = ", minModuleSize,
                    ", hmax = ", hmax, 
                    ", and deepSplit = ", deepSplit, ".", sep = "")
-                   
+  
   
   
   ########
@@ -796,7 +840,11 @@ ramclustR<- function(  xcmsObj=NULL,
     dimnames(ramclustObj$SpecAbund)[[2]]<-ramclustObj$cmpd
     if(!is.null(ms) & normalize=="quantile") {dimnames(ramclustObj$SpecAbund)[[1]]<-tmpnames1[[1]]}
     if(!usePheno | is.null(xcmsObj)) {dimnames(ramclustObj$SpecAbund)[[1]]<-dimnames(ramclustObj$MSdata)[[1]]} 
-    if(usePheno & !is.null(xcmsObj)) {dimnames(ramclustObj$SpecAbund)[[1]]<-as.vector(xcmsObj@phenoData[,1])[msfiles]}
+    if(usePheno & !is.null(xcmsObj)) {
+      if(!newXCMS) {dimnames(ramclustObj$SpecAbund)[[1]]<-as.vector(xcmsObj@phenoData[,1])[msfiles]}
+      if(newXCMS) {dimnames(ramclustObj$SpecAbund)[[1]]<-as.vector(xcmsObj@phenoData[[1]])[msfiles]}
+    }
+    
     g<-Sys.time()
   }
   
